@@ -24,6 +24,7 @@
 #include "tp-ifaces-signals-marshal.h"
 #include "tp-props-iface-gen.h"
 #include "tp-props-iface.h"
+#include "internal.h"
 
 
 #define TP_TYPE_PROPERTY_DESCRIPTION (dbus_g_type_get_struct ("GValueArray", \
@@ -75,7 +76,7 @@ struct _TpPropsPrivate
 {
   gboolean properties_ready;
 
-  int mappings_len;
+  guint mappings_len;
   PropertyMapping *mappings;
 };
 
@@ -128,7 +129,7 @@ static void tp_props_iface_dispose(GObject *obj)
 static void tp_props_iface_finalize(GObject *obj)
 {
   TpPropsIface *self = TELEPATHY_PROPS_IFACE(obj);
-  int i;
+  guint i;
   for (i=0; i < PRIV(self)->mappings_len; i++)
     {
       if (PRIV(self)->mappings[i].value)
@@ -334,7 +335,7 @@ void tp_props_iface_set_mapping (TpPropsIface *self,
 gboolean tp_props_iface_get_value (TpPropsIface* self, guint prop_id,
                                   GValue *return_value)
 {
-  int i;
+  guint i;
 
   if (!PRIV (self)->properties_ready)
     return FALSE;
@@ -381,7 +382,7 @@ gboolean tp_props_iface_set_value (TpPropsIface* self, guint prop_id,
                                   const GValue *value)
 {
   /*TODO add option for an error callback*/
-  int i;
+  guint i;
   GPtrArray *props;
 
   if (!PRIV(self)->properties_ready)
@@ -413,11 +414,30 @@ gboolean tp_props_iface_set_value (TpPropsIface* self, guint prop_id,
   return FALSE;
 }
 
+TelepathyPropertyFlags
+tp_props_iface_property_flags (TpPropsIface *self,
+                               guint prop_id)
+{
+  guint i;
+
+  if (!PRIV (self)->properties_ready)
+    return 0;
+
+  for (i = 0; i < PRIV (self)->mappings_len; i++)
+    {
+      if (PRIV (self)->mappings[i].user_id == prop_id)
+        {
+          return PRIV (self)->mappings[i].flags;
+        }
+    }
+
+  return 0;
+}
 
 static void
 set_properties_values (TpPropsIface *self, GPtrArray *properties)
 {
-  int i,j;
+  guint i, j;
 
   for (i = 0; i < properties->len; i++)
     {
@@ -475,8 +495,7 @@ properties_got_cb (DBusGProxy *proxy, GPtrArray *properties, GError *error, gpoi
 
   if (error)
     {
-      g_debug ("getting properties failed: %s (%s)", error->message,
-        dbus_g_error_get_name (error));
+      _tp_warn_failure ("getting properties", error);
       g_error_free (error);
       return;
     }
@@ -496,7 +515,7 @@ property_flags_changed_cb (DBusGProxy *proxy, GPtrArray *properties,
 {
   TpPropsIface *self = TELEPATHY_PROPS_IFACE (user_data);
   GArray *get_props;
-  int i, j;
+  guint i, j;
 
   if (!PRIV(self)->properties_ready)
     return;
@@ -509,7 +528,7 @@ property_flags_changed_cb (DBusGProxy *proxy, GPtrArray *properties,
       GValue property = {0};
       guint32 id, flags;
 
-      g_value_init (&property, TP_TYPE_PROPERTY_CHANGE);
+      g_value_init (&property, TP_TYPE_PROPERTY_FLAGS_CHANGE);
       g_value_set_static_boxed (&property, g_ptr_array_index (properties, i));
       dbus_g_type_struct_get (&property, 0, &id, G_MAXUINT);
 
@@ -543,15 +562,14 @@ static void
 properties_listed_cb (DBusGProxy *proxy, GPtrArray *properties, GError *error, gpointer user_data)
 {
   TpPropsIface *self = TELEPATHY_PROPS_IFACE (user_data);
-  int i,j;
+  guint i, j;
   guint32 id, flags;
   gchar *name;
   GArray *get_props;
 
   if (error)
     {
-      g_debug ("listing properties failed: %s (%s)", error->message,
-        dbus_g_error_get_name (error));
+      _tp_warn_failure ("listing properties", error);
       g_error_free (error);
       return;
     }
